<?php
/*
Plugin Name: Bulk User Access Editor
Description: Change access permissions for many users at once.
Version: 2.01
Requires at least: 3.66
*/

// DON'T UPDATE ANYTHING BELOW THIS LINE
addAction('admin_postlogin', 'bulkUserAccessEditor_dispatch', null, 0);
pluginAction_addHandlerAndLink("Run", 'bulkUserAccessEditor_showForm');

// dispatch commands
function bulkUserAccessEditor_dispatch() {

  // only allow admin access
  if (!$GLOBALS['CURRENT_USER']['isAdmin']) {
     return;
  }
  // only run for this plugin
  if (@$_REQUEST['_pluginName'] !== 'bulkUserAccessEditor') {
     return;
  }

  require_once $GLOBALS['PROGRAM_DIR'] . '/lib/Fields/AccessList.php';
  require_once $GLOBALS['PROGRAM_DIR'] . '/lib/viewer_functions.php';

  // dispatch plugin actions
  $action = @$_REQUEST['_pluginAction'];
  if ($action === 'showform') {
    bulkUserAccessEditor_showForm();
  }
  elseif ($action === 'update')  {
    bulkUserAccessEditor_update();
  }
  else {
    die(__FUNCTION__ . ": Unknown action");
  }

}

function bulkUserAccessEditor_showForm() {
    // prepare adminUI() placeholders
    $adminUI = [];
    // page title
    $adminUI['PAGE_TITLE'] = [
        t("Plugins")          => '?menu=admin&action=plugins',
    ];

    // button
    $adminUI['BUTTONS'] = [
        [
        'name' => 'back_to_plugins',
        'label' => 'Back to Plugins &gt;&gt;',
        'onclick' => 'window.location="?menu=admin&action=plugins";',
        'type' => 'button'
        ],
    ];
  
    // content
    $adminUI['CONTENT'] = ob_capture(function() {
    //Setup the section access list form
    $schema = loadSchema("_accesslist");
    $records = DB::query('SELECT * FROM :_accounts')->raw;
    $al = new Itools\CMSB\Fields\AccessList('somewhere', $schema, $records);
    $permissionEditor = $al->getEditComponent();

    ?>
    <input type='hidden' name='_pluginName' value='bulkUserAccessEditor' />
    <input type='hidden' name='_pluginAction' value='update' />
      <div class="col-sm-2">Users To Modify</div>
      <div class="col-sm-9" style="margin-bottom:20px">
        <div class="col-sm-4">
          <label><input type='radio' name='userGroup' value='1' /> All Users</label><br />
          <label><input type='radio' name='userGroup' value='2' /> Administrators</label><br />
          <label><input type='radio' name='userGroup' value='3' /> Non-Administrators</label><br /><br />
        </div>
        <div class="col-sm-8">
          <?php echo adminUI_button(['type' => 'submit', 'btn-type' => 'default', 'label' => t('Commit Changes'), 'value' => '1']); ?>
        </div>
      </div>
      <?php echo $permissionEditor; ?>
      <script type='text/javascript'>
        $('[name=\"accessList\\[all\\]\\[accessLevel\\]\"]').val('1');
        $('.sectionAccessList').slideDown();
      </script>
  <?php });
  // display content
  adminUI($adminUI);
  exit;
}
function bulkUserAccessEditor_update() {
    $message = "";

    $isAdmin = match(@$_REQUEST['userGroup']) {
        '1' => null,      // All users
        '2' => 1,         // Administrators
        default => 0      // Non-Administrators
    };

    if(!alert()) {
        if($backupFileName = backupDatabase(NULL, "_accesslist")){
          $message .= "Access List table backed up to " . $backupFileName;
        }
        else {
          alert("Error: Backup failed.");
        }

        if(!alert())
        {
            // Get the applicable user records
            if ($isAdmin === null) {
                $accountsRecords = DB::query("SELECT * FROM :_accounts")->raw();
            }
            else{
              $accountsRecords = DB::query("SELECT * FROM :_accounts WHERE isAdmin = :isAdmin",
                  [':isAdmin' => $isAdmin])->raw();
            }

            //update each user with the new permissions
            $usersUpdated = 0;
            if($accountsRecords) {
                $usersUpdated = _bulkUserAccessEditor_updateAccessList($accountsRecords);
            }

            $message .= "<br />Account privileges updated for $usersUpdated users.";
            notice($message);
        }
    }

    bulkUserAccessEditor_showForm();
}

/*
  Modification of _updateAccessList() from save.php
  $userNum is now accepted as a parameter, not from the $_REQUEST variable
  Copied because save.php runs other code that we don't want.
*/

function _bulkUserAccessEditor_updateAccessList($users) {
  // error checking
  if (!@$_REQUEST['accessList']) { die(__FUNCTION__ . ": No accessList specified!"); }

  // check user num
  if (!$users) {
    die(__FUNCTION__ . ": Couldn't get user number!");
  }

  // create random save id
  $randomSaveId = uniqid('', true);
  $user_num_to_delete = array();
  $userCount = 0;
  $fieldNames   = "userNum, tableName, accessLevel, maxRecords, randomSaveId";

  foreach($users as $user) {
        $userCount++;
        $userNum = (int) $user['num'];

        //Making the IN list for the delete query later
        $user_num_to_delete[] = $userNum;

        // create insert query
        foreach ($_REQUEST['accessList'] as $tableName => $tableValues) {
              // don't save section access unless "bySection" is specified for all
              if ($tableName != 'all' && $_REQUEST['accessList']['all']['accessLevel'] != '1') { continue; }
              // skip sections with no access allowed
              if ($_REQUEST['accessList']['all']['accessLevel'] == '0') { continue; }

              $maxRecords = @$tableValues['maxRecords'] != '' ? $tableValues['maxRecords'] : null;
              $insertQuery  = "INSERT INTO :__accesslist ( $fieldNames ) 
                               VALUES ( :usernum, :tablename, :accesslevel, 
                                        :maxrecords, :randomsaveid )";
              // insert new access rights
             DB::query($insertQuery, [
                  ':usernum' => $userNum,
                  ':tablename' => $tableName,
                  ':accesslevel' => $tableValues['accessLevel'],
                  ':maxrecords' => $maxRecords,
                  ':randomsaveid' => $randomSaveId
              ]);
        }
  }

      // delete old access rights
      $deleteQuery = "DELETE FROM :__accesslist WHERE userNum IN(:deletein) AND randomSaveId != :randomSaveId";
      $deleteQueryParams = [
              ':deletein' => DB::raw(implode(',',$user_num_to_delete)),
              ':randomSaveId' => $randomSaveId
      ];

      DB::query($deleteQuery, $deleteQueryParams);
      return $userCount;
  }

  // EOF
